import pygame
import serial
import cv2 as cv2
import numpy as np
from time import sleep
from matplotlib import pyplot as plt
from gpiozero import Button
from time import sleep

#Initialize GPIO Ports for the touch sensors
button_1 = Button(19)
button_2 = Button(6)

#Initialize Serial
ser = serial.Serial('/dev/ttyACM0', 115200)
user_input = '0'

minimum_pos = 0
lowest_distance = 385
pickedUp = False

def passMessage(user_input):
    output = user_input.encode('utf-8')
    ser.write(output)
    sleep(0.1)

#Initialize Pygame and Joysticks
pygame.init()
pygame.joystick.init()
joysticks = [pygame.joystick.Joystick(x) for x in range(1)]
j1 = joysticks[0]
j1.init()
#j2 = joysticks[1]
#j2.init()

#Import Template Block
template = cv2.imread('original_block.jpg',0)

#Resize Template Image
print(template.shape)
scale_percent = 9 # percent of original size
width = int(template.shape[1] * scale_percent / 100)
height = int(template.shape[0] * scale_percent / 100)
dim = (width, height)
template = cv2.resize(template, dim)
print(template.shape)

#Setup Webcam
w, h = template.shape[::-1]
cap = cv2.VideoCapture(0)

def getLowestPoint():
    highest_point_y = 480
    lowest_point_y = 0
    false_positive_y = 0
    
    #Process Frame
    success, img_rgb = cap.read()
    img_rgb = img_rgb[0:480, 100:400]
    img_gray = cv2.cvtColor(img_rgb, cv2.COLOR_BGR2GRAY)

    #Get critical values
    res = cv2.matchTemplate(img_gray,template,cv2.TM_CCOEFF_NORMED)
    threshold = 0.3
    loc = np.where( res >= threshold)
    
    
    for pt in zip(*loc[::-1]):
        if pt[0] > 33:
            #print(pt[0])
            false_positive_y = pt[1]
        elif pt[0] <= 33:
            if abs(pt[1] - false_positive_y) > 5:
                cv2.rectangle(img_rgb, pt, (pt[0] + w, pt[1] + h), (0,0,255), 2)
                if pt[1] < highest_point_y:
                    highest_point_y = pt[1]
                if pt[1] > lowest_point_y :
                    lowest_point_y = pt[1]
            
    cv2.imshow('img_rgb', img_rgb)
    range = abs(highest_point_y - lowest_point_y)
    
    return lowest_point_y, range, highest_point_y

#Begin Loop
ctr = 0
while True:  
    
    critical_value_list = getLowestPoint()
    minimum_pos = critical_value_list[0]
    range = critical_value_list[1]
    maximum_pos = critical_value_list[2]
    #print(range)
    
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break
    
    events = pygame.event.get()
    for event in events:
        if event.type == pygame.JOYBUTTONDOWN:
            print(event.dict, event.joy, event.button, "pressed")
            if (event.button == 9 or event.button == 10):
                #booleanClaw
                user_input = '0'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
            elif (event.button == 4):
                #rotateClockwise
                user_input = '1'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
            elif (event.button == 5):
                #rotateCounterclockwise
                user_input = '2'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
            elif (event.button == 0):
                #goDown
                minimum_pos = 0
                user_input = '5'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
            elif (event.button == 3):
                #goUp
                user_input = '4'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
            elif (event.button == 1):
                #goForward
                user_input = '7'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
            elif (event.button == 2):
                #goBackward
                user_input = '8'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
        elif event.type == pygame.JOYBUTTONUP:
            #print(event.dict, event.joy, event.button, "released")
            if (event.button == 4 or event.button == 5):
                #stopRotation
                user_input = '3'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
            if (event.button == 0 or event.button == 3):
                #noGo
                user_input = '6'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
        elif event.type == pygame.JOYHATMOTION:
            #print(event.dict, event.joy, event.hat, event.value)
            if (event.value == (1, 0)):
                #incrementForward
                user_input = '9'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)
            elif (event.value == (-1, 0)):
                #incrementBackward
                user_input = 'A'
                output = user_input.encode('utf-8')
                ser.write(output)
                sleep(0.1)                
                
    if ctr > 50:
        if minimum_pos < lowest_distance:
            #print("go up")
            passMessage('B')
            sleep(0.25)
        elif minimum_pos >= lowest_distance:
            #print("stay there")
            if pickedUp == False:
                passMessage('6')
                sleep(0.5)
                passMessage('4')
                sleep(5)
                passMessage('6')
                sleep(0.5)
                passMessage('7')
                sleep(0.5)
                passMessage('9')
                sleep(0.5)
                passMessage('9')
                sleep(0.5)
                passMessage('9')
                sleep(0.5)
                passMessage('0')
                sleep(0.5)
                passMessage('8')
                sleep(0.5)
                pickedUp = True
            if range > 15:
                #print("go up")
                passMessage('B')
                sleep(0.25)
            elif range <= 15:
                #print("stay there")
                if maximum_pos < 300:
                    passMessage('B')
                    sleep(0.25)
                elif maximum_pos >= 300:
                    passMessage('6')
                    sleep(0.25)
                    passMessage('4')
                    sleep(9.5)
                    passMessage('6')
                    sleep(0.5)
                    passMessage('1')
                    sleep(0.6)
                    passMessage('3')
                    sleep(0.5)
                    passMessage('7')
                    sleep(0.75)
                    passMessage('0')
                    sleep(0.5)
                    passMessage('8')
                    sleep(0.5)
                    break 
    
    ctr += 1

cap.release()
cv2.destroyAllWindows()







